package de.hdm_stuttgart.mi.sd1.test.aufgabe2;


import de.hdm_stuttgart.mi.sd1.aufgabe2.MaexleWurf;
import de.hdm_stuttgart.mi.sd1.model.Wuerfel;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import static de.hdm_stuttgart.mi.sd1.model.Wuerfel.*;

/**
 * Teste Ordnung von Würfelergebnissen beim »Mäxchen« Spiel.
 *
 */

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
@SuppressWarnings({"UnusedDeclaration"})
public class Test_MaexchenVergleich extends ExaminationTestDefaults {

  static private final int countNichtSonderfälle = (5 * 6) / 2 - 1; // (3|1), (3|2), ...

  private static final MaexleWurf[]
          nichtSonderFaelle1 = new MaexleWurf[countNichtSonderfälle],
          nichtSonderFaelle2 = new MaexleWurf[countNichtSonderfälle],

  paschFaelle = new MaexleWurf[6],
          maexchenFaelle = new MaexleWurf[2];

  static {
    int index = 0;
    for (int i = 2; i < 6; i++) { // 5 + 4 + 3 + 2 + 1 == countNichtSonderfälle
      for (int j = 0; j < i; j++) {
        nichtSonderFaelle1[index] = new MaexleWurf(Wuerfel.values()[i], Wuerfel.values()[j]);
        nichtSonderFaelle2[index] = new MaexleWurf(Wuerfel.values()[j], Wuerfel.values()[i]);
        index++;
      }
    }
    for (int i = 0; i < 6; i++) {
      paschFaelle[i] = new MaexleWurf(Wuerfel.values()[i], Wuerfel.values()[i]);
    }
    maexchenFaelle[0] = new MaexleWurf(EINS, ZWEI);
    maexchenFaelle[1] = new MaexleWurf(ZWEI, EINS);
  }

  /**
   * toString()
   */
  @Test @Marking(points = 2) public void test_80() {
    Assert.assertEquals("(DREI|EINS)", new MaexleWurf(EINS, DREI).toString());
    Assert.assertEquals("(DREI|EINS)", new MaexleWurf(DREI, EINS).toString());
  }

  /**
   * Gleichheit nicht-Sonderfälle
   */
  @Test @Marking(points = 4) public void test_100() {
    for (int i = 0; i < countNichtSonderfälle; i++) {
      Assert.assertEquals("i = " + i, 0, nichtSonderFaelle1[i].compareTo(nichtSonderFaelle1[i]));
      Assert.assertEquals("i = " + i, 0, nichtSonderFaelle2[i].compareTo(nichtSonderFaelle2[i]));

      Assert.assertEquals("i = " + i, 0, nichtSonderFaelle1[i].compareTo(nichtSonderFaelle2[i]));
      Assert.assertEquals("i = " + i, 0, nichtSonderFaelle2[i].compareTo(nichtSonderFaelle1[i]));

      Assert.assertTrue("i = " + i, 0 < nichtSonderFaelle1[i].compareTo(
              new MaexleWurf(EINS, EINS)));
      Assert.assertTrue("i = " + i, 0 < nichtSonderFaelle1[i].compareTo(
              new MaexleWurf(EINS, ZWEI)));
      Assert.assertTrue("i = " + i, 0 < nichtSonderFaelle1[i].compareTo(
              new MaexleWurf(ZWEI, EINS)));
    }

  }

  /**
   * Gleichheit Pasch
   */
  @Test @Marking(points = 2) public void test_120() {
    for (int i = 1; i <= 6; i++) {
      Assert.assertEquals("i = " + i, 0, paschFaelle[i - 1].compareTo(paschFaelle[i - 1]));
      Assert.assertTrue("i = " + i, 0 < nichtSonderFaelle1[i].compareTo(paschFaelle[i - 1]));
    }
  }

  /**
   * Gleichheit Mäxchen
   */
  @Test @Marking(points = 2) public void test_140() {
    Assert.assertEquals(0, new MaexleWurf(EINS, ZWEI).compareTo(new MaexleWurf(EINS, ZWEI)));
    Assert.assertEquals(0, new MaexleWurf(EINS, ZWEI).compareTo(new MaexleWurf(ZWEI, EINS)));
    Assert.assertEquals(0, new MaexleWurf(ZWEI, EINS).compareTo(new MaexleWurf(EINS, ZWEI)));
    Assert.assertEquals(0, new MaexleWurf(ZWEI, EINS).compareTo(new MaexleWurf(ZWEI, EINS)));
  }

  /**
   * Ungleichheit Normalfälle untereinander
   */
  @Test @Marking(points = 2) public void test_220() {
    for (int i = 0; i < countNichtSonderfälle; i++) {
      for (int j = i + 1; j < countNichtSonderfälle; j++) {
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle1[i].compareTo(nichtSonderFaelle1[j]));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle1[i].compareTo(nichtSonderFaelle2[j]));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle2[i].compareTo(nichtSonderFaelle1[j]));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle2[i].compareTo(nichtSonderFaelle2[j]));

        Assert.assertTrue("i = " + i + ", j = " + j, nichtSonderFaelle1[j].compareTo(nichtSonderFaelle1[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, nichtSonderFaelle1[j].compareTo(nichtSonderFaelle2[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, nichtSonderFaelle2[j].compareTo(nichtSonderFaelle1[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, nichtSonderFaelle2[j].compareTo(nichtSonderFaelle2[i]) < 0);
      }
    }
  }
  /**
   * Ungleichheit Pasch - Normalfall
   */
  @Test @Marking(points = 4) public void test_240() {
    for (int i = 0; i < countNichtSonderfälle; i++) {
      for (int j = 0; j < paschFaelle.length;j++) {

        final MaexleWurf p = paschFaelle[j];
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle1[i].compareTo(p));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle2[i].compareTo(p));

        Assert.assertTrue("i = " + i + ", j = " + j, p.compareTo(nichtSonderFaelle1[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, p.compareTo(nichtSonderFaelle2[i]) < 0);
      }
    }
  }

  /**
   * Ungleichheit Mäxchen - Normalfall
   */
  @Test @Marking(points = 2) public void test_260() {
    for (int i = 0; i < nichtSonderFaelle1.length; i++) {

      for (int j = 0; j < maexchenFaelle.length; j++) {

        final MaexleWurf m = maexchenFaelle[j];

        Assert.assertTrue("i = " + i + ", j = " + j, m.compareTo(nichtSonderFaelle1[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, m.compareTo(nichtSonderFaelle2[i]) < 0);

        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle1[i].compareTo(m));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle2[i].compareTo(m));
      }
    }
  }

  /**
   * Ungleichheit Mäxchen - Pasch
   */
  @Test @Marking(points = 2) public void test_280() {
    for (int i = 0; i < maexchenFaelle.length; i++) {
      final MaexleWurf m = maexchenFaelle[i];
      for (int j = 0; j < paschFaelle.length; j++) {
        final MaexleWurf p = paschFaelle[j];

        Assert.assertTrue(0 < p.compareTo(m));
        Assert.assertTrue(m.compareTo(p) < 0);
      }
    }
  }
}
