package de.hdm_stuttgart.mi.sd1.task1;

/**
 * Creating backup filenames
 */
public class B_Backup {

    /**
     * <p>Filename extension character e.g. in <code>index<span style="color:red;">.</span>html</code> </p>
     */
    static public final char EXTENSION_SEPARATOR = '.';

    /**
     * <p>Create a backup filename from a given filename and backup extension string.</p>
     *
     * <p>Rules:</p>
     *
     * <ul>
     *     <li>
     *         <p>If the given filename itself does not have an extension the backup extension string gets
     *         appended.</p>
     *     </li>
     *
     *     <li>
     *         <p>In case of an extension being denoted by a period (.) the backup extension string will
     *         be inserted prior to the filename's extension.</p>
     *     </li>
     *
     *     <li>
     *         <p>In case of multiple period characters only the very last one denotes an extension's beginning.</p>
     *     </li>
     *
     * </ul>
     *
     * <p>We provide some samples:</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Creating a backup filename</caption>
     *     <tbody>
     *         <tr>
     *             <th>filename</th>
     *             <th>backupExtension</th>
     *             <th>return</th>
     *         </tr>
     *         <tr>
     *             <td><code>testfilename</code></td>
     *             <td><code style='color:red;'>_bak</code></td>
     *             <td><code>testfilename<span style='color:red;'>_bak</span></code></td>
     *         </tr>
     *         <tr>
     *             <td><code>register.docx</code></td>
     *             <td><code style='color:red;'>.bak</code></td>
     *             <td><code>register<span style='color:red;'>.bak</span>.docx</code></td>
     *         </tr>
     *         <tr>
     *             <td><code>.cfgfile</code></td>
     *             <td><code style='color:red;'>.bak</code></td>
     *             <td><code><span style='color:red;'>.bak</span>.cfgfile</code></td>
     *         </tr>
     *         <tr>
     *             <td><code>file.c.q.xml</code></td>
     *             <td><code style='color:red;'>-backup</code></td>
     *             <td><code>file.c.q<span style='color:red;'>-backup</span>.xml</code></td>
     *         </tr>
     *     </tbody>
     * </table>
     *
     * <section class="implementationHints">
     *     <h4 class="implementationHints">Hints:</h4>
     *
     *     <ul>
     *         <li>{@link String#lastIndexOf(int)} allows for searching the last occurrence of the filename period
     *         separator character <code>'.'</code>.</li>
     *
     *         <li>{@link String#substring(int, int)} allows for selecting portions of a string by means of from and to
     *         index values.</li>
     *     </ul>
     *
     * </section>
     *
     * @param filename A non-null file name string.
     * @param backupExtension The desired extension string to be inserted / appended.
     *
     * @return The backup file's name.
     */
    static public String createBackupFilename(final String filename, final String backupExtension) {
        return "blahblahblah"; // Implement me correctly
    }

    private B_Backup(){/* Ignore me: My sole purpose is suppressing default constructor javadoc generation */}

}
